!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
!! Subroutin program (Ken-ichi Amano)                       !!
!! Nelder-Mead subroutine program for 1D array (1D function)!! 
!!                                                          !!
!! The number of the variables is nump                        !!
!! The dimension of NM method is (generally) nump+1           !! 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Please serch "changeable", you can find changeable parameters.   
! nump: the number of the variables: Input 
! nump+1: the number of the nodes for ameba: Input
! tot: tot(nump+1,nump): Input
! err: err(nump+1): Input
! cjv1: convergence judge value 1 for grandminimum point: Input
! cjv2: convergence judge value 2 for metastable point: Input
! ans: ans(nump): Output 

! See "nump" in the module, which is used 
! in the both main and subroutine programs.

! User may change the module name "example" when
! this suhbroutine is installed to user's program.

! User must prepare subtoutine error(a,b)
! Using a newly generated a, 
! error value of b (e.g., error sum of squares) must be calculated.

! User must prepare "tot" and "err" before this subroutine.
! User must set values of "cjv1" and "cjv2" for convergence judgements.

! Re-calling this subroutine program, user can perform NM method
! more than once. Then, user can get a better result. 

! If user select kani = 4, prepare proper initial guesses,
! and see areas of "Fabrication of the multi amebas".

![kani] parameter reference
!4: Multi ameba annealed NM method

!For speed up, user can reduce the shrinking time using random number
!Check face (*o*)


!In the subroutine, err(i) values are basically positive.
!If the value becomes negative, a severe problem may occur.
!Please care about it, especially when you make the err subroutine.
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      module mixing
      integer(8),parameter :: nump=9 !number of the spline "movable" points
      real(8) :: cRE,cEX,cCO1,cCO2,cSH,cSH2 !common
      real(8) :: defpara !common 
      real(8) :: def !common
      integer(8) :: ipena !forbidded area exist (1) or not (0)
      integer(8) :: nab,nab0,nabR!do loop variable for nameba (nab=1,nameba)
      integer(8) :: kwhile !kwhile DO loop number in NM method
      integer(8),parameter :: kwmax=200000 !max value of kwhile do loop
      integer(8),parameter :: kwmemo=1000 !writing frequency  !noww
      integer(8),parameter :: kwmemo2=1000 !writing frequency of ITERATION ERROR

      integer(8),parameter :: kani=4 !select the type of NM method
      integer(8),parameter :: nameba=1 !if user selects kani = 4 or 6, enter the number of amebas
      real(8),parameter :: penalty=1.0d0 !if use selects kani = 5 or 6, input a certain penalty for forbidden area
      real(8),parameter :: clambda=0.d0 !penalty coefficient for errsum, see subroutine error.

      real(8),parameter :: cjv1=0.d0 !set the convergence judge value 1 
      real(8),parameter :: cjv2=1.d-7 !set the convergence judge value 2 noww
      real(8),parameter :: cjv2pre=dsqrt(cjv2) !set this, if user set kani = 3 for inverse shrink
      real(8),parameter :: cjv3=1.d-3 !refer 2*(convSSR with nameba=10 in kani = 4)
!++++++++++++++++++++++++++++++++++++++++++++++++++
      integer(8),parameter :: nx=2048*1 !KinoshitaFFT
      real(8),parameter :: pai=dacos(-1.d0) !Circular constant 3.1415
      integer(8),parameter :: ngrid=nx !mdiis
      integer(8),parameter :: nsub=2 !mdiis
      
      !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      real(8),parameter :: endp=10.d0 !end point of the spline (unit length = ds) !noww
      real(8),parameter :: wpm=0.80d0 !start point of the effective wall
      real(8),parameter :: ampran=0.10d0 !amplitude for random height change of pot u noww
      integer(8),parameter :: nmlmm=idint(dfloat(nump)/2.d0) !the number of max local max and min
      integer(8),parameter :: ntotsu=nump !the number of outotsu
      !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      real(8),dimension(nx) :: qk,hexp
      real(8),dimension(nx) :: us
      real(8),dimension(nameba) :: ansSlog

      real(8) :: rho
      real(8) :: ds
      real(8) :: dx,dk
      real(8) :: w,xx
c      integer(8) :: judc
      integer(8) :: HNCPY
      integer(8) :: nexp
      integer(8) ::iterr

      real(8) :: df_ist_ien,xsta
      real(8) :: aveverpre,aveverpre8
      integer(8) :: ista,iend
      end module mixing
!********************************************************************
!********************************************************************
!********************************************************************
      program main
      use mixing
      real(8),dimension(nump+1,nump) :: tot
      real(8),dimension(nump+1) :: err
      real(8),dimension(nump) :: ans
      real(8),dimension(nump) :: tmp

      real(8) :: delta
      real(8) :: verb,verb2
      real(8) :: rx ! for random number
      real(8) :: det,detmax ! ameba matrix determinant

      integer(8) :: i,j,k,kx
      integer(8) :: maxl,minl,nshr

      allocatable :: totmax(:,:)
      allocate(totmax(nump+1,nump))

!=== Parameter defpara setting =====

      !From 0(not changed) to 0.5 (flatten): normally 0.2-0.4
      defpara=0.30d0 

c Gao's method (ANMS) algorithm with adaptive parameters
      cRE=1.d0 !reflection coefficient (normally 1, generally fixed)
      cEX=1.d0+(2.d0/dfloat(nump)) !expansion coefficient (normally 2)
      cCO1=0.75d0-(1.d0/(2.d0*dfloat(nump))) !contraction coefficient 1 (normally 0.5)
      cCO2=0.75d0-(1.d0/(2.d0*dfloat(nump))) !contraction coefficient 2 (normally 0.5)
      cSH=1.d0-1.d0/dfloat(nx) !shrinking coefficient (normally 0.5)

c      cRE=1.d0 !reflection 
c      cEX=2.d0  !expansion
c      cCO1=0.8d0 !outer contraction (1)
c      cCO2=0.8d0 !inner contraction (2)
c      cSH=0.5d0  !shrink [from 0 to 1]

      cSH2=1.d0-cSH !obtain cSH2
!*************************************************
!***** read input files **************************

      read(5,*)rho    !(Number Density)*ds^3
      read(5,*)ds !ds(nm)
      read(5,*)dx
      read(5,*)w,xx
      read(5,*)HNCPY !(1=HNC, 2=PY, 3=VM, 4=VCG, 5=KH)

      dk=pai/(nx*dx)
      nab=1;nab0=0 !initial 

      open(537,file="Sq")
      open(679,file="hexp")
      do k=1,nx
      read(537,*,end=735)qk(k),hexp(k)
      hexp(k)=(hexp(k)-1.d0)/rho
      write(679,*)k,hexp(k)
      end do
  735 nexp=k-1
      print *,"the point number in Sq: nexp=",nexp
      close(537)
      close(679)
!***************************************************
!***** obtain ista,iend,xsta,df_ist_ien ************
      do i=1,nx !search starting point for spline
      x=dabs(dfloat(i)*dx)
      if(x.ge.1.d0*wpm)then
      xsta=x
      ista=i
      exit
      end if
      end do

      do i=ista,nx !search end point for spline
      x=dabs(dfloat(i)*dx)
      if(x.gt.endp)then
      iend=i
      exit
      end if
      end do
 
      ista=idnint(1.d0/dx)
      iend=iend-1
      df_ist_ien=dfloat(iend-ista)
!*********************************************************!
!*********************************************************!
!*** INVERSE PROBLEM OF SAXS *****************************!

! The exact ans are 1, 2, 3, ... , nump
      tot=0.d0 !initial setting
      err=0.d0 !initial setting
      ans=0.d0 !initial setting
       
      
      call initialtot(tot) !Preparation of the tot
   
      aveverpre=0.d0
      do i=1,nump+1
       tmp(:)=tot(i,:)
         call error(tmp,verb)
       err(i)=verb !preparation of err
       print "(i11,x,g13.5,'prepared 1st ameba')",i,verb
       aveverpre=aveverpre+verb
      end do
      aveverpre=aveverpre/dfloat(nump+1)
      aveverpre8=aveverpre*8.d0 !8 timed (2*2*2)
      print *,"aveverpre=",aveverpre
      print *,"aveverpre8=",aveverpre8

      if(isnan(aveverpre) .eqv. .TRUE.)then
      print *,"Calculation stopped, so you must run it again"
      stop
      end if
    
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      print *,"NM convergence start"
      call Nelder_Mead_0(tot,err,ans)
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!*** END OF THE INVERSE PROBLEM OF SAXS *******************!      
!**********************************************************!
!**********************************************************!

        print *,"(1) See hcal_all_range_ave (not hcal_ans)"
        print *,"(2) See pot_u_all_range_ave (not u_ans)."

  174 continue
      end program main  
   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine Nelder_Mead_0(tot,err,ans)
      use mixing
      real(8),dimension(nump+1,nump) :: tot
      real(8),dimension(nump+1) :: err
      real(8),dimension(nump) :: ans

      integer(8) :: i,j,k,kx

      ! Multi ameba Annealed NM method
      call Nelder_Mead_4(tot,err,ans)

      end subroutine Nelder_Mead_0
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine estimation(hcal,verb,verb_a,verb_b)
      use mixing
      real(8),dimension(nexp) :: hcal
      real(8) :: verb,verb_a,verb_b
      real(8) :: errsum
      integer(8) :: i,j,k

      errsum=0.d0
      DO k=1,nexp
      errsum=errsum+(hcal(k)-hexp(k))**2.d0 !sum of squared residuals: SSR
      END DO
      verb_a=errsum/dfloat(nexp)

        errsum=0.d0
        do i=ista,iend
        errsum=errsum+(us(i+1)-us(i-1))**2.d0 !slope
c        errsum=errsum+((us(i+2)-us(i))-(us(i)-us(i-2)))**2.d0 !curveture
        end do
        verb_b=errsum/df_ist_ien 

        verb=verb_a+clambda*verb_b

      end subroutine estimation
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine error(buf,verb)
      use mixing
      real(8),dimension(nump) :: buf
      real(8),dimension(nexp) :: hcal 
      real(8) :: verb,verb_a,verb_b
      real(8) :: errsum
      integer(8) :: i,j,k
      

      call stOZclosure(buf,hcal)
 
      call estimation(hcal,verb,verb_a,verb_b)
       
      if(iterr .eq. 1)verb=verb*2.d0 !for ITERATION ERROR in OZ-closure

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      if(mod(kwhile,kwmemo) .eq. 0)then
      open(457,file="hcal")
      do k=1,nexp 
      write(457,*)k,hcal(k)
      end do
      close(457)
      end if
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    

      end subroutine error

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine error_forb(buf,verb)
      use mixing
      real(8),dimension(nump) :: buf
      real(8) :: verb
      real(8) :: errsum
      integer(8) :: i,ia,ib,ics2,icc2
      real(8) :: cons1,cons2,cons3,cons4,slope2,curve2
!For user: add penalty, then the forbidden area is made.
  
      !not change this line
      ipena=0 !now safe, if ipena is changed to 1, the penalty is added.


!- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
! Check the buf position, whether forbidden or not.
! This area is changeable depending on your rule.
! Comment for SAXS: check safe or out by using 
!                  [q^-2 rule] and/or [S(0) inequality rule]. 

       ics2=0
       icc2=0
       ia=idnint(wpm/dx)+1
       ib=dnint(endp/dx)-1
       do i=ia,ib
        cons1=us(i-1)-us(i-2)
        cons2=us(i)-us(i-1)
        cons3=us(i+1)-us(i)
        cons4=us(i+2)-us(i+1)
           slope2=cons2*cons3
           curve2=(cons2-cons1)*(cons4-cons3)

         if(slope2 .le. 0.d0)then
         ics2=ics2+1 !;print *,"slope2"
         end if
         if(curve2 .le. 0.d0)then
         icc2=icc2+1 !;print *,"curve2"
         end if

          if(ics2.gt.nmlmm .and. icc2.gt.ntotsu)then
          ipena=2
          exit
          else if(ics2 .gt. nmlmm)then
          ipena=1
          else if(icc2 .gt. ntotsu)then
          ipena=1         
          end if
       end do
!- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 


! Add penalty (position of grand minimum is not changed) don't worry 
!"penalty" and "def" are common parameters, don't worry.
! Comment for SAXS: Normally, 
! there is no need to change equation below
      verb=verb+dfloat(ipena)*penalty*def

      end subroutine error_forb
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine final_error(buf,verb)
      use mixing
      real(8),dimension(nump) :: buf
      real(8),dimension(nexp) :: hcal
      real(8) :: verb,verb_a,verb_b
      real(8) :: errsum
      integer(8) :: i,j,k


      call stOZclosure(buf,hcal)
      call estimation(hcal,verb,verb_a,verb_b)

        print *, " "
        print *,"FINAL (obtained from the averaged potential)"
        print *,"******* final errsum of SSR *************"
        print *,"SSR=",verb_a
        print *, " "
        print *,"***** final errsum of pure penalty value *******"
        print *,"clambda=",clambda
        print *,"PURE PENALTY=",verb_b
        print *," "

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      open(457,file="hcal_ans")
      do k=1,nexp
      write(457,*)k,hcal(k)
      end do
      close(457)
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    

      end subroutine final_error
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine semifinal_error(ansS)
      use mixing
      real(8),dimension(nameba,nump) :: ansS
      real(8),dimension(nexp) :: hcal
      real(8),dimension(nx) :: ha0,ha1,ca0,ua0,halog
      real(8),dimension(nump) :: buf
      real(8) :: verb,verb_a,verb_b
      real(8) :: errsum
      integer(8) :: i,j,k
      
      real(8),allocatable :: convSSR(:),convPENA(:),convSUM(:)
      allocate (convSSR(nameba),convPENA(nameba),convSUM(nameba))

      ha0=0.d0
      ha1=0.d0
      j=0
      DO nab=1,nameba!---------------------------------
      buf(:)=ansS(nab,:)

      nab0=nab;nabR=2 !important setting for subroutine pot
      call semistOZclosure(buf,hcal,ha1)
      call estimation(hcal,verb,verb_a,verb_b)

c        print *," "
c        print *,"nab=",nab
c        print *,"******* final errsum of SSR *************"
c        print *,"SSR=",verb_a

c        print *,"***** final errsum of pure penalty value *******"
c        print *,"PURE PENALTY=",verb_b
c        print *,"clambda=",clambda
c        print *," "

c        print *,"***** final errsum of SSR + penalty *******"
c        print *,"SSR + penalty=",verb
c        print *," "

      if(iterr .eq. 0)then
      j=j+1
      halog(:)=halog(:)+ha1(:)
      end if
c      END DO!-----------------------------------------

      print *,"the accepted ameba",j,"/",nameba
c      print *," "
      ha0(:)=halog(:)/dfloat(j)
      do k=1,nexp
      hcal(k)=ha0(k)
      end do


      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      if(nab .eq. nameba)then
      open(657,file="hcal_all_range_ave")
      do k=1,nx
      write(657,*)k,ha0(k)
      end do
      close(657)
      end if
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      ca0(:)=ha0(:)/(rho*ha0(:)+1.d0)
      IND=-1
      call FT(IND,ha0)
      call FT(IND,ca0)
      
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      if(nab .eq. nameba)then
      open(unit=65,file="h-out_all_range_ave")
      do i=1,nx
      x=dfloat(i)*dx
      write(65,*)x,ha0(i)
      end do
      close(65)
      end if
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      !for HNC
      ua0(:)=(ha0(:)+1.d0)/dexp(ha0(:)-ca0(:))
!      ua0(:)=ha0(:)-ca0(:)-dlog(ha0(:)+1.d0)
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      if(nab .eq. nameba)then
      open(91,file="pot-u_all_range_ave")
      do i=1,nx
      x=dfloat(i)*dx
      write(91,*)x,ua0(i)
      end do   
      close(91)
      end if
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      us(:)=ua0(:)
      call estimation(hcal,verb,verb_a,verb_b)   
        if(nab .eq. nameba)then
        print *," "
        print *,"total nameba=",nameba
        print *,"======= final errsum of <SSR> ============="
        print *,"SSR=",verb_a

        print *,"====== final errsum of <pure penalty value> ======"
        print *,"PURE PENALTY=",verb_b
        print *,"clambda=",clambda
        print *," "

        print *,"===== final errsum of <SSR + penalty> ======="
        print *,"SSR + penalty=",verb
        print *," "
        end if
       
       convSSR(nab)=verb_a
       convPENA(nab)=verb_b
       convSUM(nab)=verb

      END DO ! -----------------------------------  conv conv conv
! * * * * * * * * * * * * * * * * * * * * * 
      open(291,file="convSSR")
      open(292,file="convPENA")
      open(293,file="convSUM")
      do nab=1,nameba
      write(291,*)nab,convSSR(nab)
      write(292,*)nab,convPENA(nab)
      write(293,*)nab,convSUM(nab)
      end do
      close(291)
      close(292)
      close(293)
      print *,"+ + + CHECK THE FILES: convSSR,convPENA,convSUM + + +"
      deallocate (convSSR,convPENA,convSUM)
      end subroutine semifinal_error
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine Nelder_Mead_4(tot,err,ans)
      use mixing
      real(8),dimension(nump+1,nump) :: tot
      real(8),dimension(nump+1,nump) :: totmake
      real(8),dimension(nump+1) :: err
      real(8),dimension(nump) :: ans

      real(8) :: delta
      real(8) :: verb,verb2
      real(8) :: rx ! for random number
      real(8) :: vmax,vmin
      real(8) :: Pverb1,Pverb2

      integer(8) :: i,j,k,kx
      integer(8) :: maxl,minl,nshr

      real(8),allocatable :: ave(:),buf(:),ext(:)
      real(8),allocatable :: res(:)
      real(8),allocatable :: ansS(:,:),totS(:,:,:)

      allocate (ave(nump),buf(nump),ext(nump))
      allocate (res(nump+1))
      allocate (ansS(nameba,nump))
      allocate (totS(nameba,nump+1,nump))
      ave=0.d0; buf=0.d0; ext=0.d0
      ansS=0.d0
      totS=0.d0

      dfloatnump=dfloat(nump) !for speed up

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Main process of Nelder-Mead method starts !!!!!!!!!!!!!!!!
      
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !Fabrication of the multi amebas!!!!!!!!!!!!!!!!!!!!! 
           ! Below is an example !
      DO i=1,nameba 
        if(i .eq. 1)then
          Do j=1,nump+1
            do k=1,nump
            totS(i,j,k)=tot(j,k)
            end do
          End do
        else
        call initialtot(totmake) !Preparation of the other tot
          Do j=1,nump+1
            do k=1,nump
            totS(i,j,k)=totmake(j,k)
            end do
          End do
        end if
      END DO
      !User must fabricate many ameobas before calculation
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!@-@-
      DO nab=1,nameba !THE LARGE DO LOOP STARTS (nab)
      print *,"ameba number #",nab
      if(nab .ne. 1)then
      do i=1,nump+1
         tot(i,:)=totS(nab,i,:)
         buf(:)=tot(i,:)
           call error(buf,verb)
           err(i)=verb !reset land scape: not annealed here         
      end do
      end if
!*+*+
  852 continue
      DO kwhile=1,kwmax !THE BIG DO LOOP STARTS (kwhile)
      nshr=0 !Shrinked or not, recorded parameter, 0 or 1

!     Obtain ave(nump) before anneald NM mehotd
      vmax=maxval(err,1)
      vmin=minval(err,1)
!- - - - - - - - - - - - - -
c      delta=2.d0*(vmax-vmin)/(dabs(vmax+vmin)) !convergence judge A
      delta=vmax-vmin !for convergence judge (B)

      if(mod(kwhile,kwmemo) .eq. 0)then
      print '(i11,x,3g13.5)',kwhile,vmax,vmin,delta !print: convergence flow
      end if
!
      IF(vmin .lt. cjv1)THEN
      print *,"convergence is finished by cjv1"
      EXIT
      END IF

      IF(vmin .gt. aveverpre8)THEN !2018-10-27
      iterr=1;print *,"iterr=1,aveverpre8"
      EXIT
      END IF

      IF(delta .lt. cjv2)THEN
      print *,"convergence is finished by cjv2"
         if(vmin .lt. cjv3)then !cjv3 for convSSR,convPENA,convSUM
         print *,"convergence is finished by cjv3"
         EXIT
         else
         iterr=1;print *,"iterr=1,in"
         EXIT
         end if
      END IF
!- - - - - - - - - - - - - - -
      def=(vmax-vmin)*defpara

        res(:)=err(:) !reservation

        !(1) sharp & flat  annealing
        do i=1,nump+1 !First annealing
        err(i)=err(i)+def*random(rx)
        end do

c        !(2) flat annealing
c        do i=1,nump+1 !First annealing
c        errpre=err(i)+def*random(rx)
c          if(errpre .lt. vmin)then
c          err(i)=vmin+(vmin-errpre)
c          else if(errpre .gt. vmax)then
c          err(i)=vmax-(errpre-vmax)
c          else
c          err(i)=errpre
c          end if
c        end do

         maxl=maxloc(err,1) !Re-consideration of maxl
         minl=minloc(err,1) !Re-consideration of minl
!:::::
        ave=0.d0
        DO kx=1,nump
         do j=1,nump+1
          IF(j .ne. maxl)THEN
          ave(kx)=ave(kx)+tot(j,kx)
          END IF
         end do
         ave(kx)=ave(kx)/dfloatnump
        END DO

!REFLECTION*********************************************************
      buf(:)=ave(:)+cRE*(ave(:)-tot(maxl,:)) !"Reflection"
      call error(buf,verb) !To user: Please calculate an error value
                           !using a newly generated "buf" function,
                           !and substitute the error value into "verb".
                 !If err(minl) =< verb =< err(maxl), Reflection is done.
      Pverb1=verb
      verb=verb+def*random(rx)
!Expansion***********************************************************
      IF(verb .lt. err(minl))THEN
      ext(:)=ave(:)+cEX*(buf(:)-ave(:)) !"Expansion"
                   !cEX is changeable parameter.

      call error(ext,verb2)  !To user: Please calculate an error value
                             !using a newly generated "ext" function,
                             !and substitute the error value into
                             !"verb2".
      Pverb2=verb2
      verb2=verb2+def*random(rx)

          if(verb2 .lt. verb)then
!          print *,"Expansion is done"
          buf(:)=ext(:)
          verb=verb2
          Pverb1=Pverb2
          else
!          print *,"Reflection is done (upper)"
          end if

!Conctraction1&2+Shrink*************************************************
      ELSE IF(verb .ge. err(maxl))THEN  !changeable
            !e.g., 1.5d0*err(maxl) or 2.d0*err(maxl)

      ext(:)=ave(:)+cCO1*(buf(:)-ave(:))  !"Contraction 1(outer)"
                   !cCO1 is changeable.

       call error(ext,verb2) !To user: Please calculate an error value
                             !using a newly generated "ext" function,
                             !and substitute the error value into
                             !"verb2".
      Pverb2=verb2
      verb2=verb2+def*random(rx)

          if(verb2 .lt. err(maxl))then
!          print *,"Contraction 1 is done"
          buf(:)=ext(:)
          verb=verb2
          Pverb1=Pverb2
!>>>>>>>>>>>>>>>>>>>

          else
          ext(:)=ave(:)-cCO2*(buf(:)-ave(:))  !"Contraction 2 (inner)"
                       !cCO2 is changeable.

          call error(ext,verb2) !To user: Please calculate an error value
                                !using a newly generated "ext" function,
                                !and substitute the error value into
                                !"verb2".
      Pverb2=verb2
      verb2=verb2+def*random(rx)
             if(verb2 .lt. err(maxl))then
!             print *,"Contraction 2 is done"
             buf(:)=ext(:)
             verb=verb2
             Pverb1=Pverb2
!>>>>>>>>>>>>>>>>>>>

             else
!             print *,"Shrinking is done"
               nshr=1 !shrinking operation is recorded to nshr
               do j=1,nump+1
                if(j .ne. minl)then
                   tot(j,:)=cSH*tot(j,:)+cSH2*tot(minl,:) !"Shrinking"
                                                     !cSH is changeable
                   buf(:)=tot(j,:)

                call error(buf,verb) !To user: Please calculate an error value
                           !using a newly generated "buf" function,
                           !and substitute the error value into "verb".
                err(j)=verb !def changed by shrinking: not annealed here

                end if
               end do
             end if
          end if
      ELSE
!      print *,"Reflection is done (lower)"
      END IF
!**** Reflection,Expansion,Contraction,Shrinking (END) ****
!**********************************************************


!**********************************************************
!*** Make err & new tot (if not shrinked just before)******
      if(nshr .eq. 0)then
      err(:)=res(:)
      err(maxl)=Pverb1 !reset land scape: not annealed here
      tot(maxl,:)=buf(:)
      else
      err(minl)=res(minl)
      end if
!***
      END DO ! THE BIG DO LOOP ENDS (kwhile)

        if(kwhile-1.eq.kwmax .or. iterr.eq.1)then 
c        print *,"The present ansewer is not added to ansS, and"
c        print *,"the previous answer is substituted into ansS."
c        ansS(nab,:)=ansS(nab-1,:)
        print *,"***** Restart *****"
        nabR=1

          call initialtot(totmake) !Preparation of the new tot
          Do j=1,nump+1
            do k=1,nump
            totS(nab,j,k)=totmake(j,k)
            end do
          End do

        do i=1,nump+1
         tot(i,:)=totS(nab,i,:)
         buf(:)=tot(i,:)
           call error(buf,verb)
           err(i)=verb !reset land scape: not annealed here
        end do
        goto 852
        else
        minl=minloc(err,1)     
        ansS(nab,:)=tot(minl,:)
        end if
   
      END DO ! THE LARGE DO LOOP ENDS (nab)
!!!  Main process of Nelder-Mead method ends  !!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      call semifinal_error(ansS)

      deallocate(ave,buf,ext)
      deallocate(res)
      deallocate(ansS)
      deallocate(totS)
      end subroutine Nelder_Mead_4

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      function random(rx) !random number function (-1 to +1) 
      use mixing
      real(8) :: x
      integer(8) :: iseedsize,IT
      allocatable :: iseed(:)
      call random_seed(size=iseedsize)
!      iseedsize=100 !For gfortran
! When gfortran, comment-out "call random_seed(size=iseedsize))".
      allocate(iseed(iseedsize))
      call random_seed(get=iseed)
      call SYSTEM_CLOCK(COUNT=IT)
      iseed=IT !It is also important process for generation
      call random_seed(put=iseed)
      call random_number(x)
      random=(x-0.5d0)*2.d0 !make random number - plus minus 1
      deallocate(iseed)
      return
      end
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      function basepot(rleng) !fabricate base potential 
      use mixing
      real(8) :: rleng
      real(8) :: cbp
      real(8) :: ranjud
      save ranjud !It indicates the initial base potential.

      IF(nab .ne. nab0)THEN
      nab0=nab
      ranjud=(random(rx)+1.d0)/2.d0  !from 0 to 1
      ansSlog(nab)=ranjud

      ELSE IF(nabR .eq. 1)THEN
      nabR=0
      ranjud=(random(rx)+1.d0)/2.d0  !from 0 to 1
      ansSlog(nab)=ranjud

      ELSE IF(nabR .eq. 2)THEN
      nabR=0
      ranjud=ansSlog(nab) !used for subroutine semifinal_error
      END IF

        cbp=1.d0/rleng
        coef1=dmod(ranjud*11111.d0,10.d0)/10.d0
        coef2=dmod(ranjud*33333.d0,10.d0)/10.d0
        coef3=dmod(ranjud*55555.d0,10.d0)/10.d0
        coef4=dmod(ranjud*77777.d0,10.d0)/10.d0
        coef5=dmod(ranjud*99999.d0,10.d0)/10.d0

        if(ranjud .lt. 1.d0/5.d0)then
          basepot=(coef1+3.d0)*(cbp**10.d0) !repulsive 10-4 LJ !noww

        else if(ranjud .lt. 2.d0/5.d0)then
          basepot=(coef2+3.d0)*((cbp**10.d0)-(cbp**4.d0)) !full 10-4 LJ

        else if(ranjud .lt. 3.d0/5.d0)then
          basepot=(coef3+3.d0)*(cbp**10.d0) !repulsive 10-4 LJ &
     &           +coef4*cbp*dexp(-(coef5+1.5d0)*(rleng-1.d0)) !repulsive Yukawa

        else if(ranjud .lt. 4.d0/5.d0)then
          basepot=(coef1+3.d0)*(cbp**10.d0) !repulsive 10-4 LJ &
     &           -coef2*cbp*dexp(-(coef3+1.5d0)*(rleng-1.d0)) !attractive Yukawa

        else
          basepot=(coef1+3.d0)*(cbp**10.d0) !repulsive 10-4 LJ &
     &           +coef2*cbp*dexp(-(coef3+1.5d0)*(rleng-1.d0)) !repulsive Yukawa     
     &           -coef4*cbp*dexp(-(coef5+1.5d0)*(rleng-1.d0)) !attractive Yukawa

        end if



      end function basepot
!**************************************************************************
!**************************************************************************
!**************************************************************************
      subroutine stOZclosure(buf,hcal)
      use mixing
      implicit double precision(a-h,o-z)

      dimension eta0(nx)
      dimension u(nx),eta(nx),c(nx),h(nx)
      dimension buf(nump)
      dimension hcal(nexp) 

      call OZclosure(eta0,eta,u,c,h,buf)

!********* Preparation of hcal(k) ***************** 
c      call writing2(eta)
      call hnc2(eta,c,h,u)
      call writing3(c,h)  !change h(nx) => h(nexp) 
          do k=1,nexp
          hcal(k)=h(k)
          end do
!**************************************************      
    
      end subroutine stOZclosure
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      subroutine semistOZclosure(buf,hcal,ha1)
      use mixing
      implicit double precision(a-h,o-z)

      dimension eta0(nx)
      dimension u(nx),eta(nx),c(nx),h(nx)
      dimension buf(nump)
      dimension ha1(nx)
      dimension hcal(nexp)

      call OZclosure(eta0,eta,u,c,h,buf)

!********* Preparation of ha1 *****************
c      call writing2(eta)
      call hnc2(eta,c,h,u)
      call writing3(c,h)  !change h(nx) => h(nexp)
        ha1(:)=h(:)
          do k=1,nexp
          hcal(k)=h(k)
          end do
!**************************************************

      end subroutine semistOZclosure
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!=================================================================
      subroutine OZclosure(eta0,eta,u,c,h,buf)
      use mixing
      implicit double precision(a-h,o-z)
      dimension eta0(nx)
      dimension u(nx),eta(nx),c(nx),h(nx)
      dimension buf(nump)
   
      iterr=0
      call pot(u,buf)

! BIG LOOP STARTS ::::::::::::::::::::::::::::
      eta=0.d0
      DO 2000 m=1,10000

      do i=1,nx
      eta0(i)=eta(i)
      end do

      call hnc(eta,c,h,u)

      IND=1
      call FT(IND,c)

      do k=1,nx
      eta(k)=rho*c(k)**2.d0/(1.d0-rho*c(k))  !rho
      end do

      IND=-1
      call FT(IND,eta)

      diffmax=0.d0
      do i=1,nx
       diff=dabs(eta(i)-eta0(i))
        if(diff .gt. diffmax)then
        diffmax=diff
       end if
      end do

c      print *, m, diffmax
      if(diffmax .gt. 1.d+3)then
         if(mod(kwhile,kwmemo2) .eq. 0)then
         print *, "ITERATION ERROR"
         end if 
c      stop     !just now
      iterr=1   !just now
      GOTO 2001 !just now
      end if

      IF(diffmax .lt. w)THEN
      GOTO 2001
      ELSE
!      DO i=1,nx
!      eta(i)=(1.0d0-xx)*eta0(i)+xx*eta(i)
!      END DO
      call mdiis1d(eta0,eta,xx,xx,xx)
      END IF

 2000 continue
! BIG LOOP ENDS ::::::::::::::::::::::::::::::
 2001 continue

      end subroutine OZclosure
!*****************************************************************
!*****************************************************************
!*****************************************************************
      subroutine pot(u,buf)
      use mixing
      implicit double precision(a-h,o-z)
      dimension u(nx)
      dimension buf(nump)
      dimension Xa(nump+2),Ya(nump+2)

c      judc=0  ! For subroutine "hnc2"
c      db=1.d-9 !samll length value

! Make potential curve with cubic spline

!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      sepspl=(endp-1.d0*wpm)/dfloat(nump+1)
      !Start from Xa(1) & Ya(1), then ...
      Xa(1)=xsta
      Ya(1)=basepot(xsta) !artificial wall height

      dnump2=dfloat(nump+2)
      aa=xsta !Xa=aa*exp(bb*(j-1)) gradual increase of Xa
      bb=dlog(endp/aa)/(dnump2-1.d0)
      do j=2,nump+1
!      Xa(j)=xsta+dfloat(j-1)*sepspl !constant separation
      Xa(j)=aa*dexp(bb*dfloat(j-1)) !gradually changing separation
      Ya(j)=basepot(Xa(j))+buf(j-1)
      end do

      Xa(nump+2)=xsta+dfloat(nump+1)*sepspl
      Ya(nump+2)=0.d0 !a boundary condition
!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

      call TSEVAL(Xa,Ya) !Cubic spline subroutine starts 
                         !There is terminal smoothing in us(i)

      do i=1,nx
      u(i)=dexp(-us(i))
      end do

      if(mod(kwhile,kwmemo) .eq. 0)then     
      call writing1(u)
      end if

      end subroutine pot
**********************************************************************
**********************************************************************
      subroutine hnc(eta,c,h,u)
      use mixing
      implicit double precision(a-h,o-z)
      dimension eta(nx),c(nx),h(nx),u(nx)

      !! subroutine hnc calculates c and h from eta and u.
      !! u(i) has already been in exponential (see subroutine pot).

      alpha=1.0175d0-0.275d0*rho

      IF(HNCPY .eq. 1)THEN !HNC
        do i=1,nx
        c(i)=u(i)*dexp(eta(i))-eta(i)-1.d0
        h(i)=eta(i)+c(i)
        end do
      ELSEIF(HNCPY .eq. 2)THEN !PY
        do i=1,nx
        c(i)=u(i)*(1.d0+eta(i))-eta(i)-1.d0
        h(i)=eta(i)+c(i)
        end do
      ELSEIF(HNCPY .eq. 3)THEN !VM
        do i=1,nx
      c(i)=u(i)*dexp(eta(i)-(eta(i)**2.d0)/(2.d0+1.6d0*eta(i)))
     &    -eta(i)-1.0d0
      h(i)=eta(i)+c(i)
        end do
      ELSEIF(HNCPY .eq. 4)THEN !VCGM
        do i=1,nx

           if(u(i) .gt. 1.d-10)sf=eta(i)+dlog(u(i))
           if(u(i) .le. 1.d-10)sf=eta(i)
           kk=idnint(1.d0/dx) !This if is important for outputing
           if(i .eq. kk)then  !h-out @ VCGM @ subroutine [hnc2]
           if(u(i) .gt. 1.d-10)sf=eta(i)+dlog(u(i)*2.d0)
           end if
             IF(sf .ge. 0.d0)THEN
      c(i)=u(i)*dexp(eta(i)-(sf**2.d0)/(2.d0+2.d0*alpha*sf))
     &    -eta(i)-1.0d0
             ELSE
      c(i)=u(i)*dexp(eta(i)-(sf**2.d0)/2.d0)-eta(i)-1.0d0
             END IF
      h(i)=eta(i)+c(i)
        end do
      ELSEIF(HNCPY .eq. 5)THEN !KH
         do i=1,nx
         fkh=u(i)*dexp(eta(i))
         if(fkh .gt. 1.d0)then
            c(i)=dlog(u(i)) !KH
         else
            c(i)=u(i)*dexp(eta(i))-eta(i)-1.d0 !KH
         end if
         h(i)=eta(i)+c(i)
         end do
      END IF

      end subroutine hnc
!=================================================
      subroutine hnc2(eta,c,h,u)
      use mixing
      implicit double precision(a-h,o-z)
      dimension eta(nx),c(nx),h(nx),u(nx)

      !! subroutine hnc calculates c and h from eta and u.
      !! u(i) has already been in exponential (see subroutine pot).

      alpha=1.0175d0-0.275d0*rho

      IF(HNCPY .eq. 1)THEN !HNC
        do i=1,nx
        c(i)=u(i)*dexp(eta(i))-eta(i)-1.d0
        h(i)=eta(i)+c(i)
        end do
      ELSEIF(HNCPY .eq. 2)THEN !PY
        do i=1,nx
        c(i)=u(i)*(1.d0+eta(i))-eta(i)-1.d0
        h(i)=eta(i)+c(i)
        end do
      ELSEIF(HNCPY .eq. 3)THEN !VM
        do i=1,nx
      c(i)=u(i)*dexp(eta(i)-(eta(i)**2.d0)/(2.d0+1.6d0*eta(i)))
     &    -eta(i)-1.0d0
      h(i)=eta(i)+c(i)
        end do
      ELSEIF(HNCPY .eq. 4)THEN !VCGM
        do i=1,nx

           if(u(i) .gt. 1.d-10)sf=eta(i)+dlog(u(i))
           if(u(i) .le. 1.d-10)sf=eta(i)
           kk=idnint(1.d0/dx)
           if(i .eq. kk)then !important for outputing h-out @ VCGM
           if(u(i) .gt. 1.d-10)sf=eta(i)+dlog(u(i)*2.d0)
           end if

             IF(sf .ge. 0.d0)THEN
      c(i)=u(i)*dexp(eta(i)-(sf**2.d0)/(2.d0+2.d0*alpha*sf))
     &    -eta(i)-1.0d0
             ELSE
      c(i)=u(i)*dexp(eta(i)-(sf**2.d0)/2.d0)-eta(i)-1.0d0
             END IF
      h(i)=eta(i)+c(i)
        end do
      ELSEIF(HNCPY .eq. 5)THEN !KH
         do i=1,nx
         fkh=u(i)*dexp(eta(i))
         if(fkh .gt. 1.d0)then
            c(i)=dlog(u(i)) !KH
         else
            c(i)=u(i)*dexp(eta(i))-eta(i)-1.d0 !KH
         end if
         h(i)=eta(i)+c(i)
         end do
      END IF


c      h(0)=-1.d0
c      vsum=0.d0 !linear-response (lr)
c      do i=1,nx
c      x=dfloat(i)*dx
c      vsum=vsum
c     &    -4.d0*pai*(x**2.d0)*((h(i)+h(i-1))/2.d0)*dx !lr
c      end do
c      print *,"vsum*rho must be within 0 to 1 (amano inequality)"
c      print *,"vsum*rho=",vsum*rho !lr


c      if(judc .eq. 1)then
c      kk=idnint(1.d0/dx)
c      g=(h(kk)+1.d0)*2.0d0
c      h(kk)=g-1.d0
c      end if

      end subroutine hnc2
!=================================================
      subroutine FT(IND,X) !KinoshitaFFT
      use mixing
      implicit double precision(a-h,o-z)
      dimension X(nx)

      NT1=nx-1
      NT2=nx/2

      DEL3=dx**3
      A02=4.D0*dfloat(nx)*DEL3
      A45=0.5D0/dfloat(nx)/dfloat(nx)/DEL3


      if(IND .eq. 1)then !Forward transform *************

      DO 10 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*AJ
   10 CONTINUE

      CALL SINET(nx,NT2,C0,X)

      DO 5 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*A02/AJ
    5 CONTINUE


      else !Backward transform ****************************

      DO 17 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*AJ
   17 CONTINUE

      CALL SINET(nx,NT2,F0,X)

      DO 30 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*A45/AJ
   30 CONTINUE

      end if ! Forward or Backward transform

      end subroutine FT
!====================================================================
!====================================================================
      SUBROUTINE SINET(N,N2,F0,F)
      use mixing
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X0
      COMPLEX*16 X(nx*2)
      DIMENSION F(nx*2),Y(nx*2)
      Y0=0.D0
      AN=DFLOAT(N)
      DO 1 J=1,N2-1
      NJ=N-J
      AJ=DFLOAT(J)
      AJN=AJ/AN
      Y1=DSIN(pai*AJN)*(F(J)+F(NJ))
      Y2=(F(J)-F(NJ))/2.D0
      Y(J)=Y1+Y2
      Y(NJ)=Y1-Y2
    1 CONTINUE

      Y(N2)=2.D0*F(N2)

      CALL REALT(N2,Y0,Y,X0,X)

      F0=0.D0
      F(1)=DREAL(X0)/2.D0

      DO 2 J=1,N2-1
      J2=J*2
      F(J2)=DIMAG(X(J))
      F(J2+1)=F(J2-1)+DREAL(X(J))
    2 CONTINUE

      END SUBROUTINE SINET
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
      SUBROUTINE REALT(N2,F0,F,X0,X)
      use mixing
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X(nx),Z(nx),C1(nx)
      COMPLEX*16 X0,CA,CB
      DIMENSION F(nx*2)

      CA=(0.5D0,-0.5D0)
      CB=(0.5D0,0.5D0)
      A=2.D0*pai/dfloat(nx)

      DO 1 J=1,N2-1
      AJ=DFLOAT(J)
      AJ=AJ*A
      B=DCOS(AJ)
      C=DSIN(AJ)
      C1(J)=DCMPLX(-C,B)/2.D0
    1 CONTINUE


      Z(1)=DCMPLX(F0,F(1))
      DO 11 J=2,N2
      J1=J-1
      J2=2*J1
      Z(J)=DCMPLX(F(J2),F(J2+1))
   11 CONTINUE

      CALL FFT(N2,1,Z)

      X0=Z(1)*CA+DCONJG(Z(1))*CB
      DO 2 J=1,N2-1
      X(J)=(0.5D0-C1(J))*Z(J+1)+(0.5D0+C1(J))*DCONJG(Z(N2-J+1))
    2 CONTINUE
      END SUBROUTINE REALT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      SUBROUTINE FFT(N,IND,X)
      use mixing
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X(nx)
      COMPLEX*16 TEMP,THETA
      J=1
      DO 1 I=1,N
      IF(I.GE.J)GO TO 2
      TEMP=X(J)
      X(J)=X(I)
      X(I)=TEMP
    2 CONTINUE
      M=N/2
    3 CONTINUE
      IF(J.LE.M)GO TO 4
      J=J-M
      M=M/2
      IF(M.GE.2)GO TO 3
    4 CONTINUE
      J=J+M
    1 CONTINUE
      KMAX=1
    5 CONTINUE
      IF(KMAX.GE.N)RETURN
      ISTEP=KMAX*2
      DO 6 K=1,KMAX
      AINDK=DFLOAT(IND*(K-1))
      AKMAX=DFLOAT(KMAX)
      THETA=DCMPLX(0.D0,PAI*AINDK/AKMAX)
      DO 7 I=K,N,ISTEP
      J=I+KMAX
      TEMP=X(J)*CDEXP(THETA)
      X(J)=X(I)-TEMP
      X(I)=X(I)+TEMP
    7 CONTINUE
    6 CONTINUE
      KMAX=ISTEP
      GO TO 5
      END SUBROUTINE FFT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*
      subroutine writing1(u)
      use mixing
      implicit double precision(a-h,o-z)
      dimension u(nx)

      open(unit=12,file="pot-u_NM")
      do i=1,nx
      x=dfloat(i)*dx
      write(12,*)x,u(i)
      end do
      close(12)

      end subroutine writing1
!=========================================
      subroutine writing2(eta)
      use mixing
      implicit double precision(a-h,o-z)
      dimension eta(nx)

      open(unit=13,file="eta-out")
      do i=1,nx
      x=dfloat(i)*dx
      write(13,*)x,eta(i)
      end do
      close(13)

      end subroutine writing2
!=========================================
      subroutine writing3(c,h)
      use mixing
      implicit double precision(a-h,o-z)
      dimension c(nx),h(nx)

c      open(unit=14,file="c-out")
c      do i=1,nx
c      x=dfloat(i)*dx
c      write(14,*)x,c(i)
c      end do
c      close(14)

c      open(unit=15,file="h-out")
c      do i=1,nx
c      x=dfloat(i)*dx
c      write(15,*)x,h(i)
c      end do
c      close(15)

!* * * * *
c      if(judc .eq. 1)then
c      kk=idnint(1.d0/dx)
c      g=(h(kk)+1.d0)/2.0d0
c      h(kk)=g-1.d0
c      end if

      IND=1
      call FT(IND,h)

c      open(unit=33,file="h-hat")
c      open(unit=34,file="h-hat-q2-rule")
c      open(unit=35,file="Sq_cal")
c      do i=1,nx
c      q=pai*dfloat(i)/(dfloat(nx)*dx*ds) !for Sawazumi (SAXS)
c      write(33,*)q,h(i)   !h(i)*q**2.d0
c      write(34,*)q,h(i)*(q**2.d0)
c      write(35,*)q,h(i)*rho+1.d0 !S(q) structure factor
c      end do
c      close(33)
c      close(34)
c      close(35)
c      print *,"0 < S(0) =< 1"
c      print *,"S(0)",2.d0*(h(1)*rho+1.d0)-(h(2)*rho+1.d0)

      end subroutine writing 3
!==========================================
      subroutine writingU(buf)
      use mixing
      implicit double precision(a-h,o-z)
      dimension u(nx)
      dimension buf(nump)
      dimension Xa(nump+2),Ya(nump+2)
      real(8) :: verb

! Make potential curve with cubic spline

      sepspl=(endp-1.d0*wpm)/dfloat(nump+1)
!Xa & Ya start from array number 1
      Xa(1)=xsta
      Ya(1)=basepot(xsta) !artificial wall height

      dnump2=dfloat(nump+2)
      aa=xsta !Xa=aa*exp(bb*(j-1)) gradual increase of Xa
      bb=dlog(endp/aa)/(dnump2-1.d0)

      end subroutine writingU
!=========================================
      subroutine initialtot(tot)
      use mixing
      implicit double precision(a-h,o-z)
      real(8),dimension(nump+1,nump) :: tot

      do i=1,nump+1
       do j=1,nump
       tot(i,j)=ampran*random(rx) !preparation of tot from far
       end do
      end do

      end subroutine initialtot
!=========================================
c-----------------------------------------------------------
c 1-Dimensional MDIIS
! Mainly by N. Yoshida, implemented by K. Amano
! See word "mdiis" in the main program and subroutine taoinverse
! Then, you can find how to implement.
c-----------------------------------------------------------
      subroutine mdiis1d(tr,trnew,dumpmax,dumpmin,dumpnume)
      use mixing
      implicit double precision(a-h,o-z)
c      use mixing
c
c     ir        ... file number of output (STDOUT)
c     iw        ... file number of input (STDIN)
c     ngrid     ... number of grid of RDF
c     nsub      ... number of MDIIS sub-space
c     tr        ... tau bond =hr-cr
c     trnew     ... latest estimation of tau bond
c     trmdiis   ... stock of tr for MDIIS sub-space
c     rsmdiis   ... stock of residual for MDIIS sub-space
c     rsdmat    ... residual overlap matrix
c     nds       ... flag for initialize of MDIIS procedure
c     idc       ... counter for MDIIS sub-space
c     irmsmin   ... index of rmsmin
c     dumpmax   ... maximum value of dumping parameter eta
c     dumpmin   ... manimum value of dumping parameter eta
c     dumpnume  ... numerator of dumping function
c

      dimension tr(ngrid),trnew(ngrid)
      dimension trmdiis(ngrid,nsub),rsmdiis(ngrid,nsub)
      dimension rsdmat(nsub+1,nsub+1),dmat(nsub+1,nsub+1),x(nsub+1)
      save trmdiis !amano added
      save rsmdiis !amano added
      save rsdmat  !amano added
      save nds,idc,irmsmin !amano added
      save rmsmin,rmsnew !amano added
c
c     --- parameter of check residual ---
      facres=10.d0
c     -----------------------------------
c
c     --- Initialize MDIIS
c
      if(nds .eq. 0)then
      nds=1
         idiiscount=0
         rsdmat(1,1)=0.d0
         do i=2,nsub+1
            do j=2,nsub+1
               rsdmat(i,j)=0.d0
            enddo
            rsdmat(i,1)=-1.d0 !linked(A)
            rsdmat(1,i)=-1.d0 !linked(A)
         enddo
      endif


      x(1)=-1.d0 !linked(A)
      do i=2,nsub+1
         x(i)=0.d0
      enddo
c
c     --- Calculate New RMS of Residual
c
      rmsnew=0.d0
      do k=1,ngrid
         rmsnew=rmsnew+(tr(k)-trnew(k))**2
      enddo
      rmsnew=dsqrt(rmsnew/dble(ngrid)) ! dble = dfloat
c
c     --- Dumping Parameter Setting
c
c     --For dynamical control of dumping factor
cc      eta=dumpnume/rmsnew
cc      if (eta.gt.dumpmax) eta=dumpmax
cc      if (eta.lt.dumpmin) eta=dumpmin
c
c     -- For static control of dumping factor
      eta=dumpmin
c
c     --- Check the lowest rms
c
      if (idc.eq.0) then
      rmsmin=rmsnew
!      print *,"idc = 0"
      end if

      if (rmsnew.le.rmsmin) then
         irmsmin=1
         rmsmin=rmsnew
      endif

c
c     --- If Convergence is Poor, Restart MDIIS
!         If Restart, 3*3 matrix is used for MDIIS
      if ((rmsnew.gt.rmsmin*facres).and.(idc.gt.0)) then
         idc=1
         sum=0.d0
         do k=1,ngrid
            sum=sum+rsmdiis(k,irmsmin)*rsmdiis(k,irmsmin)
            trmdiis(k,2)=trmdiis(k,irmsmin)
            rsmdiis(k,2)=rsmdiis(k,irmsmin)
         enddo
         rsdmat(3,3)=sum
         irmsmin=1
         rmsmin=sum
      endif

c
c     --- Increment Counter of MDIIS
!     At the first process, 2*2 matrix is used for MDIIS.
!     In the other stage, matrix is larger than or equal to 3*3
      idc=idc+1
      if (idc.gt.nsub) idc=nsub
c
c     --- Set New Residual and New tr
c
      do k=1,ngrid
         trmdiis(k,1)=trnew(k)
         rsmdiis(k,1)=trnew(k)-tr(k)
      enddo

c
c     --- Make Residual Overlap Matrix
c
      do i=1,idc
         sum=0.d0
         do k=1,ngrid
            sum=sum+rsmdiis(k,1)*rsmdiis(k,i)
         enddo
         rsdmat(i+1,2)=sum
         rsdmat(2,i+1)=sum
      enddo
c
c     --- Solve MDIIS Matrix Equation
c
      do i=1,nsub+1
         do j=1,nsub+1
            dmat(i,j)=rsdmat(i,j)
         enddo
      enddo

      eps=1.d-16
      ind=0
      det=0.d0
c      < Use Numerical Recipes>
!      call matsolv(dmat,x,idc+1,nsub+1,1) !get c from dmat and x
      call taoinverse(dmat,x,idc+1,nsub+1) !amano added
!      print *,"Lagrangian multiplier =",x(1)

c
c     --- Make New Estimate
c
      do k=1,ngrid
         sum=0.d0
         do m=1,idc
            sum=sum+x(m+1)*(trmdiis(k,m)+eta*rsmdiis(k,m))
         enddo
         trnew(k)=sum !amano changed from tr(k) to trnew(k)
      enddo
c
c     --- Shift Data trmdiis, rsmdiis and rsdmat
c
      do i=nsub,2,-1
         do j=2,nsub
            rsdmat(i+1,j+1)=rsdmat(i,j)
         enddo
      enddo

      do m=nsub,2,-1
         do k=1,ngrid
            trmdiis(k,m)=trmdiis(k,m-1)
            rsmdiis(k,m)=rsmdiis(k,m-1)
         enddo
      enddo
      irmsmin=irmsmin+1
c
c     --- Reset minimum point (if minimum point flow out)
c
      if (irmsmin.gt.nsub) then
         rmin=rsdmat(2,2)
         irmsmin=1
         do i=3,nsub+1
            if (rsdmat(i,i).lt.rmin) then
               rmin=rsdmat(i,i)
               irmsmin=i
            endif
         enddo
      endif
c---------------------------
      end subroutine mdiis1d
!**********************************************************
!==========================================================
      subroutine taoinverse(dmat,xvec,N,N0)
      implicit double precision(a-h,o-z)
      dimension dmat(N0,N0)
      dimension xvec(N0),cvec(N0)
      dimension A(N,N),X(N,N),CONF(N,N),B(N,N)
      dimension INDX(N)

!+ + + + + + + + + + + + + + + +
      do i=1,N
      do j=1,N
      A(i,j)=dmat(i,j)
      end do
      end do
!+ + + + + + + + + + + + + + + +

      B(:,:)=A(:,:)

      call MIGS(A,N,X,INDX)

      CONF(:,:)=matmul(X,B)

!      write(*, '(10e9.1)') (B(i, 1:N), i = 1,N) !previous mat
!      PRINT *,"*************************************"
!      write(*, '(10e9.1)') (X(i, 1:N), i = 1,N) !inversed mat
!      PRINT *,"*************************************"
!      write(*, '(10e9.1)') (CONF(i, 1:N), i = 1,N) !wil be unit mat

!+ + + + + + + + + + + + + + + + +
      cvec=0.d0
      do i=1,N
      ans=0.d0
      do j=1,N
      ans=ans+X(i,j)*xvec(j)
      end do
      cvec(i)=ans
      end do
      xvec(:)=cvec(:)
!+ + + + + + + + + + + + + + + + +
      end subroutine taoinverse
!- - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      SUBROUTINE MIGS (A,N,X,INDX)
!
! Subroutine to invert matrix A(N,N) with the inverse stored
! in X(N,N) in the output.  Copyright (c) Tao Pang 2001.
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      INTEGER, INTENT (IN) :: N
      INTEGER :: I,J,K
      INTEGER, INTENT (OUT), DIMENSION (N) :: INDX
      REAL(8), INTENT (IN), DIMENSION (N,N):: A
      REAL(8), INTENT (OUT), DIMENSION (N,N):: X
      REAL(8), DIMENSION (N,N) :: B
!
      DO I = 1, N
        DO J = 1, N
          B(I,J) = 0.d0
        END DO
      END DO
      DO I = 1, N
        B(I,I) = 1.d0
      END DO
!
      CALL ELGS (A,N,INDX)
!
      DO I = 1, N-1
        DO J = I+1, N
          DO K = 1, N
            B(INDX(J),K) = B(INDX(J),K)-A(INDX(J),I)*B(INDX(I),K)
          END DO
        END DO
      END DO
!
      DO I = 1, N
        X(N,I) = B(INDX(N),I)/A(INDX(N),N)
        DO J = N-1, 1, -1
          X(J,I) = B(INDX(J),I)
          DO K = J+1, N
            X(J,I) = X(J,I)-A(INDX(J),K)*X(K,I)
          END DO
          X(J,I) =  X(J,I)/A(INDX(J),J)
        END DO
      END DO
      END SUBROUTINE MIGS
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
      SUBROUTINE ELGS (A,N,INDX)
!
! Subroutine to perform the partial-pivoting Gaussian elimination.
! A(N,N) is the original matrix in the input and transformed matrix
! plus the pivoting element ratios below the diagonal in the output.
! INDX(N) records the pivoting order.  Copyright (c) Tao Pang 2001.
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      INTEGER, INTENT (IN) :: N
      INTEGER :: I,J,K,ITMP
      INTEGER, INTENT (OUT), DIMENSION (N) :: INDX
      REAL(8) :: C1,PI,PI1,PJ
      REAL(8), INTENT (INOUT), DIMENSION (N,N) :: A
      REAL(8), DIMENSION (N) :: C
!
! Initialize the index
!
      DO I = 1, N
        INDX(I) = I
      END DO
!
! Find the rescaling factors, one from each row
!
      DO I = 1, N
        C1= 0.d0
        DO J = 1, N
          C1 = DMAX1(C1,DABS(A(I,J)))
        END DO
        C(I) = C1
      END DO
!
! Search the pivoting (largest) element from each column
!
      DO J = 1, N-1
        PI1 = 0.d0
        DO I = J, N
          PI = DABS(A(INDX(I),J))/C(INDX(I))
          IF (PI .GT. PI1) THEN
            PI1 = PI
            K   = I
          ENDIF
        END DO
!
! Interchange the rows via INDX(N) to record pivoting order
!
        ITMP    = INDX(J)
        INDX(J) = INDX(K)
        INDX(K) = ITMP
        DO I = J+1, N
          PJ  = A(INDX(I),J)/A(INDX(J),J)
!
! Record pivoting ratios below the diagonal
!
          A(INDX(I),J) = PJ
!
! Modify other elements accordingly
!
          DO K = J+1, N
            A(INDX(I),K) = A(INDX(I),K)-PJ*A(INDX(J),K)
          END DO
        END DO
      END DO
!
      END SUBROUTINE ELGS
!==========================================================
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! [MEMO BY AMANO]
! This may be the natural cubic spline with cuvature
! factors 0 at the both boundary condition.
! There is no need to prepare the input data X
! with equal intervals.
! The programs written by small letters are
! additional programs by Amano.

      SUBROUTINE TSEVAL(X,Y) !cubic spline subroutine starts
      use mixing
      parameter(N=nump+2)

      real*8 X(N), Y(N), B(N), C(N), D(N), U, V, Wa, Za
      integer*8 i,j,k,iendT,jend,kend


c         open(10,file="spline_input")
c         do i=1,N
c         write(10,*)X(i),Y(i)
c         end do
c         close(10)

c      Wa=X(N)-X(1) !memorize separation, beccause X changes by "SPLINE"
      call SPLINE (N,X,Y,B,C,D)

c         open(11,file="spline_out")
c         iendT=100
c         Wa=Wa/dfloat(iendT)
c         do i=0,iendT
c         U=X(1)+Wa*dfloat(i)
c         Za=SEVAL(N,U,X,Y,B,C,D)
c         Za=Za*dexp(-((U/(endp-0.5d0))**20.d0)) !smoothing
c         write(11,*)U,Za
c         end do
c         close(11)

      do i=1,nx
      Wa=dabs(dfloat(i)*dx)
        if(Wa .lt. 1.d0*wpm)then
        us(i)=1.d+10
        else if(Wa.ge.1.d0*wpm .and. Wa.le.endp)then
        U=Wa
        Za=SEVAL(N,U,X,Y,B,C,D)
        Za=Za*dexp(-((U/(endp-0.5d0))**20.d0)) !terminal smoothing
        us(i)=Za
        else
        us(i)=0.d0
        end if
      end do

      END SUBROUTINE TSEVAL
!****************************************************************************
      FUNCTION SEVAL (N,U,X,Y,B,C,D)
!------------------------------------------------------------------------
!     EVALUATE A CUBIC SPLINE INTERPOLATION OF A DISCRETE FUNCTION F(X),
!     GIVEN IN N POINTS X(I), Y(I). THE B, C AND D COEFFICIENTS DEFINING
!     THE BEST CUBIC SPLINE FOR THE GIVEN POINTS, ARE CALCULATED BEFORE
!     BY THE SPLINE SUBROUTINE.
!
!     INPUTS:
!     N       NUMBER OF POINTS OF CURVE Y = F(X)
!     U       ABSCISSA OF POINT TO BE INTERPOLATED
!     X,Y     TABLES OF DIMENSION N, STORING THE COORDINATES
!             OF CURVE F(X)
!     B,C,D   TABLES STORING THE COEFFICIENTS DEFINING THE
!             CUBIC SPLINE
!
!     OUTPUTS:
!     SEVAL   INTERPOLATED VALUE
!             = Y(I)+DX*(B(I)+DX*(C(I)+DX*D(I)))
!             WITH DX = U-X(I), U BETWEEN X(I) AND X(I+1)
!
!     REFERENCE :
!     FORSYTHE,G.E. (1977) COMPUTER METHODS FOR MATHEMATICAL
!     COMPUTATIONS. PRENTICE-HALL,INC.
!------------------------------------------------------------------------
      REAL *8 B(N),C(N),D(N),X(N),Y(N),U,DX,SEVAL
      DATA I/1/

!     BINARY SEARCH

      IF (I.GE.N) I = 1
      IF (U.LT.X(I)) GO TO 10
      IF (U.LE.X(I+1)) GO TO 30
   10 I = 1
      J = N+1
   20 K = (I+J)/2
      IF (U.LT.X(K)) J = K
      IF (U.GE.X(K)) I = K
      IF (J.GT.I+1) GO TO 20

!     SPLINE EVALUATION

   30 DX = U-X(I)
      SEVAL = Y(I)+DX*(B(I)+DX*(C(I)+DX*D(I)))
      RETURN
      END

      SUBROUTINE SPLINE (N,X,Y,B,C,D)
!---------------------------------------------------------------------
!     THIS SUBROUTINE CALCULATES THE COEFFICIENTS B,C,D OF A CUBIC
!     SPLINE TO BEST APPROXIMATE A DISCRETE FUNCTION GIVEN BY N POINTS
!
!     INPUTS:
!     N       NUMBER OF GIVEN POINTS
!     X,Y     VECTORS OF DIMENSION N, STORING THE COORDINATES
!             OF FUNCTION F(X)
!
!     OUTPUTS:
!     A,B,C   VECTORS OF DIMENSION N, STORING THE COEFFICIENTS
!             OF THE CUBIC SPLINE
!
!     REFERENCE:
!     FORSYTHE,G.E. (1977) COMPUTER METHODS FOR MATHEMATICAL
!     COMPUTATIONS. PRENTICE-HALL,INC.
!---------------------------------------------------------------------
      IMPLICIT REAL *8 (A-H,O-Z)
      DIMENSION B(N),C(N),D(N),X(N),Y(N)
      NM1 = N-1
      IF (N.LT.2) RETURN
      IF (N.LT.3) GO TO 50

!     BUILD THE TRIDIAGONAL SYSTEM
!     B (DIAGONAL), D (UPPERDIAGONAL) , C (SECOND MEMBER)

      D(1) = X(2)-X(1)
      C(2) = (Y(2)-Y(1))/D(1)
      DO 10 I = 2,NM1
      D(I) = X(I+1)-X(I)
      B(I) = 2.D0*(D(I-1)+D(I))
      C(I+1) = (Y(I+1)-Y(I))/D(I)
      C(I) = C(I+1)-C(I)
   10 CONTINUE

!     CONDITIONS AT LIMITS
!     THIRD DERIVATIVES OBTAINED BY DIVIDED DIFFERENCES

      B(1) = -D(1)
      B(N) = -D(N-1)
      C(1) = 0.D0
      C(N) = 0.D0
      IF (N.EQ.3) GO TO 15
      C(1) = C(3)/(X(4)-X(2))-C(2)/(X(3)-X(1))
      C(N) = C(N-1)/(X(N)-X(N-2))-C(N-2)/(X(N-1)-X(N-3))
      C(1) = C(1)*D(1)*D(1)/(X(4)-X(1))
      C(N) = -C(N)*D(N-1)**2/(X(N)-X(N-3))

!     FORWARD ELIMINATION

   15 DO 20 I = 2,N
      T = D(I-1)/B(I-1)
      B(I) = B(I)-T*D(I-1)
      C(I) = C(I)-T*C(I-1)
   20 CONTINUE

!     BACK SUBSTITUTION

      C(N) = C(N)/B(N)
      DO 30 L = 1,NM1
      I = N-L
      C(I) = (C(I)-D(I)*C(I+1))/B(I)
   30 CONTINUE

!     COEFFICIENTS OF 3RD DEGREE POLYNOMIAL

      B(N) = (Y(N)-Y(NM1))/D(NM1)+D(NM1)*(C(NM1)+2.D0*C(N))
      DO 40 I = 1,NM1
      B(I) = (Y(I+1)-Y(I))/D(I)-D(I)*(C(I+1)+2.D0*C(I))
      D(I) = (C(I+1)-C(I))/D(I)
      C(I) = 3.D0*C(I)
   40 CONTINUE
      C(N) = 3.D0*C(N)
      D(N) = D(NM1)
      RETURN

!     CAS N = 2

   50 B(1) = (Y(2)-Y(1))/(X(2)-X(1))
      C(1) = 0.D0
      D(1) = 0.D0
      B(2) = B(1)
      C(2) = 0.D0
      D(2) = 0.D0
      RETURN
      END



