      module mixing
      integer(8),parameter :: nx=2048*1 !KinoshitaFFT
      real(8),parameter :: pai=dacos(-1.d0) !Circular constant 3.1415

      real(8) :: rho
      real(8) :: ds
      real(8) :: dx,dk
      real(8) :: w,xx
      integer(8) :: HNCPY

      end module mixing
!********************************************************************
!********************************************************************
!********************************************************************
      program main
      use mixing
      real(8),dimension(nx) :: ha0,ca0,ua0

      integer(8) :: i,j,k,kx

!*************************************************
!***** read input files **************************

      read(5,*)rho    !(Number Density)*ds^3
      read(5,*)ds !ds(nm)
      read(5,*)dx
      read(5,*)w,xx
      read(5,*)HNCPY !(1=HNC, 2=PY, 3=VM, 4=VCG, 5=KH)

      dk=pai/(nx*dx)
!*********************************************************!
!*********************************************************!

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      open(657,file="hcal_all_range_super_ave")
      do k=1,nx
      read(657,*)j,ha0(k)
      end do
      close(657)
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      ca0(:)=ha0(:)/(rho*ha0(:)+1.d0)
      IND=-1
      call FT(IND,ha0)
      call FT(IND,ca0)
      
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      open(unit=65,file="h-out_all_range_super_ave")
      do i=1,nx
      x=dfloat(i)*dx
      write(65,*)x,ha0(i)
      end do
      close(65)
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      !for HNC
      ua0(:)=(ha0(:)+1.d0)/dexp(ha0(:)-ca0(:))
!      ua0(:)=ha0(:)-ca0(:)-dlog(ha0(:)+1.d0)

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      open(91,file="pot-u_all_range_super_ave")
      do i=1,nx
      x=dfloat(i)*dx
      write(91,*)x,ua0(i)
      end do   
      close(91)
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   
      stop
      end
!=================================================
      subroutine FT(IND,X) !KinoshitaFFT
      use mixing
      implicit double precision(a-h,o-z)
      dimension X(nx)

      NT1=nx-1
      NT2=nx/2

      DEL3=dx**3
      A02=4.D0*dfloat(nx)*DEL3
      A45=0.5D0/dfloat(nx)/dfloat(nx)/DEL3


      if(IND .eq. 1)then !Forward transform *************

      DO 10 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*AJ
   10 CONTINUE

      CALL SINET(nx,NT2,C0,X)

      DO 5 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*A02/AJ
    5 CONTINUE


      else !Backward transform ****************************

      DO 17 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*AJ
   17 CONTINUE

      CALL SINET(nx,NT2,F0,X)

      DO 30 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*A45/AJ
   30 CONTINUE

      end if ! Forward or Backward transform

      end subroutine FT
!====================================================================
!====================================================================
      SUBROUTINE SINET(N,N2,F0,F)
      use mixing
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X0
      COMPLEX*16 X(nx*2)
      DIMENSION F(nx*2),Y(nx*2)
      Y0=0.D0
      AN=DFLOAT(N)
      DO 1 J=1,N2-1
      NJ=N-J
      AJ=DFLOAT(J)
      AJN=AJ/AN
      Y1=DSIN(pai*AJN)*(F(J)+F(NJ))
      Y2=(F(J)-F(NJ))/2.D0
      Y(J)=Y1+Y2
      Y(NJ)=Y1-Y2
    1 CONTINUE

      Y(N2)=2.D0*F(N2)

      CALL REALT(N2,Y0,Y,X0,X)

      F0=0.D0
      F(1)=DREAL(X0)/2.D0

      DO 2 J=1,N2-1
      J2=J*2
      F(J2)=DIMAG(X(J))
      F(J2+1)=F(J2-1)+DREAL(X(J))
    2 CONTINUE

      END SUBROUTINE SINET
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
      SUBROUTINE REALT(N2,F0,F,X0,X)
      use mixing
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X(nx),Z(nx),C1(nx)
      COMPLEX*16 X0,CA,CB
      DIMENSION F(nx*2)

      CA=(0.5D0,-0.5D0)
      CB=(0.5D0,0.5D0)
      A=2.D0*pai/dfloat(nx)

      DO 1 J=1,N2-1
      AJ=DFLOAT(J)
      AJ=AJ*A
      B=DCOS(AJ)
      C=DSIN(AJ)
      C1(J)=DCMPLX(-C,B)/2.D0
    1 CONTINUE


      Z(1)=DCMPLX(F0,F(1))
      DO 11 J=2,N2
      J1=J-1
      J2=2*J1
      Z(J)=DCMPLX(F(J2),F(J2+1))
   11 CONTINUE

      CALL FFT(N2,1,Z)

      X0=Z(1)*CA+DCONJG(Z(1))*CB
      DO 2 J=1,N2-1
      X(J)=(0.5D0-C1(J))*Z(J+1)+(0.5D0+C1(J))*DCONJG(Z(N2-J+1))
    2 CONTINUE
      END SUBROUTINE REALT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      SUBROUTINE FFT(N,IND,X)
      use mixing
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X(nx)
      COMPLEX*16 TEMP,THETA
      J=1
      DO 1 I=1,N
      IF(I.GE.J)GO TO 2
      TEMP=X(J)
      X(J)=X(I)
      X(I)=TEMP
    2 CONTINUE
      M=N/2
    3 CONTINUE
      IF(J.LE.M)GO TO 4
      J=J-M
      M=M/2
      IF(M.GE.2)GO TO 3
    4 CONTINUE
      J=J+M
    1 CONTINUE
      KMAX=1
    5 CONTINUE
      IF(KMAX.GE.N)RETURN
      ISTEP=KMAX*2
      DO 6 K=1,KMAX
      AINDK=DFLOAT(IND*(K-1))
      AKMAX=DFLOAT(KMAX)
      THETA=DCMPLX(0.D0,PAI*AINDK/AKMAX)
      DO 7 I=K,N,ISTEP
      J=I+KMAX
      TEMP=X(J)*CDEXP(THETA)
      X(J)=X(I)-TEMP
      X(I)=X(I)+TEMP
    7 CONTINUE
    6 CONTINUE
      KMAX=ISTEP
      GO TO 5
      END SUBROUTINE FFT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*
