      module oneDSSmod 
      implicit double precision(a-h,o-z)
      parameter(nx=2048)
      parameter(ngrid=nx) !mdiis
      parameter(nsub=3) !mdiis 
      parameter(Boltz=1.3806504d-23)
      parameter(T=298.15d0) !Temperature (K)
      parameter(e0=8.854187817d-12) !Electric Permitivity in Vaccum
      parameter(er=78.4d0) !Relative Electric Permitivity (78.4)
      parameter(Dleng=1.d-9)  !Debey length (m)
      parameter(pc=1.60217649d-19) !Proton Charge
      parameter(pai=dacos(-1.d0)) !circular constant 3.1415***
      end module oneDSSmod

      program main
      use oneDSSmod
      implicit double precision(a-h,o-z)

      dimension eta0(nx)
      dimension u(nx),eta(nx),c(nx),h(nx),hexp(nx)

      common /cb2/ rho
      common /cb3/ ds,epsi
      common /cb4/ dx,dk
      common /cb5/ judca
      common /HP/ HNCPY
      common /sp/ psi0

      read(5,*)istart
      read(5,*)rho    !(Number Density)*ds^3
      read(5,*)ds,epsi !ds(nm)
!      ds=((rho/ds)**(1.d0/3.d0))*1.d+9 !for volumefraction shell (1)
!      ds=((6.d0*0.024d0/(rho*1.d+21*pai))**(1.d0/3.d0))*1.d+9 !for density2 shell (1)
!      rho=rho*1.d+21*((ds*1.d-9)**3.d0) !for density2 shell (2)
      print *,"ds=",ds,"(nm)"
      read(5,*)psi0 !surface potential (mV)
      psi0=psi0*1.d-3 !surface potential (V)
      read(5,*)dx  
!      dx=dx*21.215688d0/ds !for volumefraction shell (2)
!      dx=dx*7.710292842d0/ds !for density2 shell (3) 
      read(5,*)w,xx
      read(5,*)HNCPY !(1=>HNC, 2=>PY, 3=>VM, 4=>VCG)
  
      dk=pai/(nx*dx)

      open(unit=35,file="Sq")
      do i=1,nx
        q=pai*dfloat(i)/(dfloat(nx)*dx*ds) !for Sawazumi (SAXS)
        expSq=expSq1(q)*dexp(-((q/0.2)**20.d0))
     &       +expSq2(q)*(1.d0-dexp(-((q/0.2)**20.d0)))
        write(35,*)q,expSq !S(q) structure factor
        hexp(i)=(expSq-1.d0)/rho
      end do
      close(35)
      print *,"0 < S(0) =< 1"


! If you want to cut the S(q) at a position and
! estimate h(r) from the cut S(q), comment-in the lines below       
      do i=1,nx
      q=pai*dfloat(i)/(dfloat(nx)*dx*ds)
      if(q >= 3.5d0)then
c      hexp(i)=0.d0
      end if
      end do


      IND=-1
      call FT(IND,hexp)

      open(unit=55,file="rough_h")
      do i=1,nx
      write(55,*)dfloat(i)*dx,hexp(i)
      end do
      close(55)
      
      end program main
!==========================================
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      function expSq1(q) !fabricate base potential 
      use oneDSSmod
      real(8) :: q

      expSq1=0.2887061678220566 + 0.015551547650476057*q
     &                   + 1.1187725594671394*q**2

      end function expSq1
!**************************************************************************
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      function expSq2(q) !fabricate base potential 
      use oneDSSmod
      real(8) :: q

      expSq2=1 + (-0.35631395251590137d0 -
     & 9.925391333641596d0*q +
     & 111.39458680071044d0*q**2 -
     & 651.3758678721845d0*q**3 +
     & 2324.43208602799d0*q**4 -
     & 5436.602962511044d0*q**5 +
     & 8847.714559334245d0*q**6 -
     & 10458.585975652439d0*q**7 +
     & 9225.423525787446d0*q**8 -
     & 6156.979871239142d0*q**9 +
     & 3114.86353420859d0*q**10 -
     & 1180.6435176000298d0*q**11 +
     & 324.35628958835065d0*q**12 -
     & 59.517926916075744d0*q**13 +
     & 5.555773538950911d0*q**14 +
     & 0.2317357730322392d0*q**15 -
     & 0.12784127391770495d0*q**16 +
     & 0.026128840971724014d0*q**17 -
     & 0.013469781044080403d0*q**18 +
     & 0.005706489362168012d0*q**19 -
     & 0.0013840770352387033d0*q**20 +
     & 0.00018449560057290604d0*q**21 -
     & 6.83491665179372d-6*q**22 -
     & 2.2332312138385623d-6*q**23 +
     & 4.890350818526922d-7*q**24 -
     & 5.091541876783966d-8*q**25 +
     & 3.1239584669051327d-9*q**26 -
     & 1.0876799472407508d-10*q**27 +
     & 1.672233117944255d-12*q**28)*
     & dexp(-((q/6.d0)**20.d0))

      end function expSq2

!=================================================
      subroutine FT(IND,X) !KinoshitaFFT
      use oneDSSmod
      implicit double precision(a-h,o-z)
      dimension X(nx)
      common /cb4/ dx,dk

      NT1=nx-1
      NT2=nx/2

      DEL3=dx**3
      A02=4.D0*dfloat(nx)*DEL3
      A45=0.5D0/dfloat(nx)/dfloat(nx)/DEL3


      if(IND .eq. 1)then !Forward transform *************

      DO 10 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*AJ
   10 CONTINUE

      CALL SINET(nx,NT2,C0,X)

      DO 5 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*A02/AJ
    5 CONTINUE


      else !Backward transform ****************************

      DO 17 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*AJ
   17 CONTINUE

      CALL SINET(nx,NT2,F0,X)

      DO 30 J=1,NT1
      AJ=DFLOAT(J)
      X(J)=X(J)*A45/AJ
   30 CONTINUE

      end if ! Forward or Backward transform

      end subroutine FT
!====================================================================
!====================================================================
      SUBROUTINE SINET(N,N2,F0,F)
      use oneDSSmod
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X0
      COMPLEX*16 X(nx*2)
      DIMENSION F(nx*2),Y(nx*2)
      Y0=0.D0
      AN=DFLOAT(N)
      DO 1 J=1,N2-1
      NJ=N-J
      AJ=DFLOAT(J)
      AJN=AJ/AN
      Y1=DSIN(pai*AJN)*(F(J)+F(NJ))
      Y2=(F(J)-F(NJ))/2.D0
      Y(J)=Y1+Y2
      Y(NJ)=Y1-Y2
    1 CONTINUE

      Y(N2)=2.D0*F(N2)

      CALL REALT(N2,Y0,Y,X0,X)

      F0=0.D0
      F(1)=DREAL(X0)/2.D0

      DO 2 J=1,N2-1
      J2=J*2
      F(J2)=DIMAG(X(J))
      F(J2+1)=F(J2-1)+DREAL(X(J))
    2 CONTINUE

      END SUBROUTINE SINET
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
      SUBROUTINE REALT(N2,F0,F,X0,X)
      use oneDSSmod
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X(nx),Z(nx),C1(nx)
      COMPLEX*16 X0,CA,CB
      DIMENSION F(nx*2)

      CA=(0.5D0,-0.5D0)
      CB=(0.5D0,0.5D0)
      A=2.D0*pai/dfloat(nx)

      DO 1 J=1,N2-1
      AJ=DFLOAT(J)
      AJ=AJ*A
      B=DCOS(AJ)
      C=DSIN(AJ)
      C1(J)=DCMPLX(-C,B)/2.D0
    1 CONTINUE


      Z(1)=DCMPLX(F0,F(1))
      DO 11 J=2,N2
      J1=J-1
      J2=2*J1
      Z(J)=DCMPLX(F(J2),F(J2+1))
   11 CONTINUE

      CALL FFT(N2,1,Z)

      X0=Z(1)*CA+DCONJG(Z(1))*CB
      DO 2 J=1,N2-1
      X(J)=(0.5D0-C1(J))*Z(J+1)+(0.5D0+C1(J))*DCONJG(Z(N2-J+1))
    2 CONTINUE
      END SUBROUTINE REALT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      SUBROUTINE FFT(N,IND,X)
      use oneDSSmod
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X(nx)
      COMPLEX*16 TEMP,THETA
      J=1
      DO 1 I=1,N
      IF(I.GE.J)GO TO 2
      TEMP=X(J)
      X(J)=X(I)
      X(I)=TEMP
    2 CONTINUE
      M=N/2
    3 CONTINUE
      IF(J.LE.M)GO TO 4
      J=J-M
      M=M/2
      IF(M.GE.2)GO TO 3
    4 CONTINUE
      J=J+M
    1 CONTINUE
      KMAX=1
    5 CONTINUE
      IF(KMAX.GE.N)RETURN
      ISTEP=KMAX*2
      DO 6 K=1,KMAX
      AINDK=DFLOAT(IND*(K-1))
      AKMAX=DFLOAT(KMAX)
      THETA=DCMPLX(0.D0,PAI*AINDK/AKMAX)
      DO 7 I=K,N,ISTEP
      J=I+KMAX
      TEMP=X(J)*CDEXP(THETA)
      X(J)=X(I)-TEMP
      X(I)=X(I)+TEMP
    7 CONTINUE
    6 CONTINUE
      KMAX=ISTEP
      GO TO 5
      END SUBROUTINE FFT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*

